@@ -1,8 +1,8 @@
 package Graph::Weighted;
 use strict;
 use Carp;
-use vars qw($VERSION); $VERSION = '0.01.1';
-use base qw(Graph);
+use vars qw($VERSION); $VERSION = '0.05.1';
+use base qw(Graph::Directed);
 
 use constant WEIGHT => 'weight';
 
@@ -18,10 +18,7 @@ sub new {  # {{{
 
     bless $self, $class;
 
-    if ($args{data}) {
-        $self->load($args{data});
-        $self->graph_weight;
-    }
+    $self->load($args{data}) if $args{data};
 
     return $self;
 }  # }}}
@@ -30,53 +27,82 @@ sub _debug {  # {{{
     print @_, "\n" if shift->{debug};
 }  # }}}
 
-sub reset_graph {  # {{{
-    my $self = shift;
-    undef $self->{data};
-    undef $self->{matrix};
-    undef $self->{heaviest_vertex};
-    undef $self->{lightest_vertex};
-    $self->delete_vertices($self->vertices);
-    $self->delete_attribute(WEIGHT);
-}  # }}}
-
 sub load {  # {{{
     my ($self, $data) = @_;
-$self->_debug("entering load with $data");
+$self->_debug('entering load with a '. ref ($data) . ' object');
 
+    if (ref ($data) eq 'Math::MatrixReal' ||
+        ref ($data) eq 'Math::Matrix'
+    ) {
+        # Math::Matrix* objects are LoLs and have the matrix as the
+        # first element.
+        $data = $data->[0];
+    }
+
+    if (ref ($data) eq 'Math::MatrixBool') {
+        # Math::MatrixBool objects are actually Bit::Vectors, that
+        # can be output by "stringification", like this:
+        # [ 1 0 0 ]
+        # [ 1 1 0 ]
+        # [ 1 1 1 ]
+        # So de-stringification gymnastics have to happen.  Ugh.
+        $data = [
+            map { [ split ] }
+                sprintf ('%s', "$data") =~ /\[\s([\d\s]+)\s\]/gs
+        ];
+    }
+
+    # We are given a HoH.
     if (ref ($data) eq 'HASH') {
-        # Set the object matrix to the HoH data.
-        $self->{matrix} = $data;
-
         # Set the vertices and weighted edges.
-        while (my ($vertex, $neighbors) = each %$data) {
-            unless (keys %$neighbors) {
+        while (my ($vertex, $successors) = each %$data) {
+            # Add an "edgeless vertex" if there are no successors or
+            # successors all have zero values.
+            my $weight = 0;
+            $weight += $_ for values %$successors;
+$self->_debug("$vertex weight = $weight");
+
+            unless (keys %$successors && ($weight || $self->{zero_edges})) {
 $self->_debug("$vertex edgeless");
                 $self->add_vertex($vertex);
             }
 
-            while (my ($neighbor, $n) = each %$neighbors) {
-$self->_debug("$vertex =($n)=> $neighbor");
-                $self->add_weighted_edge($vertex, $n, $neighbor)
+            while (my ($successor, $n) = each %$successors) {
+$self->_debug("$vertex =($n)=> $successor");
+                $self->add_weighted_edge($vertex, $n, $successor)
                     if $self->{zero_edges} || $n != 0;
             }
         }
     }
     # We are given a LoL.
     elsif (ref ($data) eq 'ARRAY') {
+        # Initialize the object data.
+        $self->{data} = {};
+
         # Set the vertices and weighted edges.
         for my $vertex (0 .. @$data - 1) {
             croak "Incorrectly sized array\n"
                 unless @{ $data->[$vertex] } == @$data;
 
-            for my $neighbor (0 .. @{ $data->[$vertex] } - 1) {
-                my $n = $data->[$vertex][$neighbor];
+            my $weight = 0;
+            $weight += $_ for @{ $data->[$vertex] };
+$self->_debug("$vertex weight = $weight");
 
-$self->_debug("$vertex =($n)=> $neighbor");
-                $self->add_weighted_edge($vertex, $n, $neighbor)
-                    if $self->{zero_edges} || $n != 0;
+            unless ($weight || $self->{zero_edges}) {
+$self->_debug("$vertex edgeless");
+                $self->add_vertex($vertex);
+                $self->{data}{$vertex} = {};
+            }
 
-                $self->{matrix}{$vertex}{$neighbor} = $n;
+            for my $successor (0 .. @{ $data->[$vertex] } - 1) {
+                my $n = $data->[$vertex][$successor];
+
+                if ($n || $self->{zero_edges}) {
+$self->_debug("$vertex =($n)=> $successor");
+                    $self->add_weighted_edge($vertex, $n, $successor);
+
+                    $self->{data}{$vertex}{$successor} = $n;
+                }
             }
         }
     }
@@ -86,8 +112,8 @@ $self->_debug("$vertex =($n)=> $neighbor");
 $self->_debug('exiting load');
 }  # }}}
 
-sub matrix {  # {{{
-    return shift->{matrix};
+sub data {  # {{{
+    return shift->{data};
 }  # }}}
 
 sub graph_weight {  # {{{
@@ -121,17 +147,17 @@ $self->_debug("weight is defined as $weight");
         my $old = $self->get_attribute(WEIGHT, $vertex);
         $self->set_attribute(WEIGHT, $vertex, $weight);
 
-        # How many neighbors does the vertex have?
-        my $n_neighbors = keys %{ $self->{matrix}{$vertex} };
+        # How many successors does the vertex have?
+        my $n_successors = keys %{ $self->{data}{$vertex} };
 
         # Distribute the weight to all outgoing edges.
-        my $new_weight = $weight / $n_neighbors;
-        for my $neighbor (keys %{ $self->{matrix}{$vertex} }) {
-            # Reset the matrix value.
-            $self->{matrix}{$vertex}{$neighbor} = $new_weight;
+        my $new_weight = $weight / $n_successors;
+        for my $successor (keys %{ $self->{data}{$vertex} }) {
+            # Reset the data value.
+            $self->{data}{$vertex}{$successor} = $new_weight;
             # Reset the outgoing edge.
-            $self->set_attribute(WEIGHT, $vertex, $neighbor, $new_weight);
-$self->_debug("$vertex =($new_weight)=> $neighbor: new vertex weight set");
+            $self->set_attribute(WEIGHT, $vertex, $successor, $new_weight);
+$self->_debug("$vertex =($new_weight)=> $successor: new vertex weight set");
         }
 
         # Adjust the total graph weight if we made a change.
@@ -145,7 +171,7 @@ $self->_debug("adjust the graph weight to $graph_weight");
         }
     }
     elsif (!$self->has_attribute(WEIGHT, $vertex)) {
-        $weight += $_ for values %{ $self->{matrix}{$vertex} };
+        $weight += $_ for values %{ $self->{data}{$vertex} };
         $weight = 0 unless $weight;
 $self->_debug("weight computed as $weight");
         $self->set_attribute(WEIGHT, $vertex, $weight);
@@ -156,20 +182,20 @@ $self->_debug('exiting vertex_weight');
 }  # }}}
 
 sub edge_weight {  # {{{
-    my ($self, $vertex, $neighbor, $weight) = @_;
-$self->_debug("entering edge_weight with $vertex and $neighbor");
+    my ($self, $vertex, $successor, $weight) = @_;
+$self->_debug("entering edge_weight with $vertex and $successor");
 
     if (defined $weight) {
 $self->_debug("weight is defined as $weight");
         # Out with the old; in with the new.
-        my $old = $self->get_attribute(WEIGHT, $vertex, $neighbor);
+        my $old = $self->get_attribute(WEIGHT, $vertex, $successor);
 
         # Reset the edge weight.
-        $self->set_attribute(WEIGHT, $vertex, $neighbor, $weight);
+        $self->set_attribute(WEIGHT, $vertex, $successor, $weight);
 
-        # Reset the matrix value.
-        $self->{matrix}{$vertex}{$neighbor} = $weight;
-$self->_debug("$vertex =($weight)=> $neighbor: new vertex weight set");
+        # Reset the data value.
+        $self->{data}{$vertex}{$successor} = $weight;
+$self->_debug("$vertex =($weight)=> $successor: new vertex weight set");
 
         # Adjust the graph and vertex weight if we made a change.
         if ($old != $weight) {
@@ -184,14 +210,14 @@ $self->_debug("adjust the vertex weight to $vertex_weight");
             $self->set_attribute(WEIGHT, $vertex, $vertex_weight);
         }
     }
-    elsif (!$self->has_attribute(WEIGHT, $vertex, $neighbor)) {
-        $weight = $self->{matrix}{$vertex}{$neighbor};
-$self->_debug("weight from the matrix is $weight");
-        $self->set_attribute(WEIGHT, $vertex, $neighbor, $weight);
+    elsif (!$self->has_attribute(WEIGHT, $vertex, $successor)) {
+        $weight = $self->{data}{$vertex}{$successor};
+$self->_debug("weight from the data is $weight");
+        $self->set_attribute(WEIGHT, $vertex, $successor, $weight);
     }
 
 $self->_debug('exiting edge_weight');
-    return $self->get_attribute(WEIGHT, $vertex, $neighbor);
+    return $self->get_attribute(WEIGHT, $vertex, $successor);
 }  # }}}
 
 sub heaviest_vertices {  # {{{
@@ -244,6 +270,20 @@ $self->_debug('exiting lightest_vertices with ['. join (', ', @{ $self->{lightes
     return $self->{lightest_vertices};
 }  # }}}
 
+sub max_weight {  # {{{
+    my $self = shift;
+    return $self->vertex_weight(
+        $self->heaviest_vertices->[0]
+    );
+}  # }}}
+
+sub min_weight {  # {{{
+    my $self = shift;
+    return $self->vertex_weight(
+        $self->lightest_vertices->[0]
+    );
+}  # }}}
+
 1;
 
 __END__
@@ -258,41 +298,47 @@ Graph::Weighted - A weighted graph implementation
 
   $g = Graph::Weighted->new(
       data => {
-          a => { b => 1, c => 2, },  # Vertices with two edges.
-          b => { a => 1, c => 3, },
-          c => { a => 2, b => 3, },
+          a => { b => 1, c => 2, },  # A vertex with two edges.
+          b => { a => 1, c => 3, },  # "
+          c => { a => 2, b => 3, },  # "
           d => { c => 1, },          # A vertex with one edge.
           e => {},                   # A vertex with no edges.
      }
   );
 
-  $x = $g->vertex_weight('a');
-  $y = $g->vertex_weight('a', $x + 1);
-
-  $x = $g->edge_weight('a', 'b');
-  $y = $g->edge_weight('a', 'b', $x + 1);
-
-  $g->reset_graph;
+  $g = Graph::Weighted->new(data => $object);  # See documentation.
 
   $g = Graph::Weighted->new();
-
   $g->load(
-      [ [ 0, 1, 2 ],
-        [ 1, 0, 3 ],
-        [ 2, 3, 0 ],
-        [ 0, 0, 1 ],
-        [ 0, 0, 0 ], ]
+      [ [ 0, 1, 2, 0, 0, ],    # A vertex with two edges.
+        [ 1, 0, 3, 0, 0, ],    # "
+        [ 2, 3, 0, 0, 0, ],    # "
+        [ 0, 0, 1, 0, 0, ],    # A vertex with one edge.
+        [ 0, 0, 0, 0, 0, ], ]  # A vertex with no edges.
   );
 
+  $data = $g->data;
+
   $w = $g->graph_weight;
 
-  $m = $g->matrix;
+  $w = $g->vertex_weight($p);
+  $w = $g->vertex_weight($p, $w + 1);
 
-  $heaviest = $g->heaviest_vertices;
-  $lightest = $g->lightest_vertices;
+  $w = $g->edge_weight($p, $q);
+  $w = $g->edge_weight($p, $q, $w + 1);
 
-  $x = $g->vertex_weight($heaviest->[$i]);
-  $y = $g->vertex_weight($lightest->[$j]);
+  $heavies = $g->heaviest_vertices;
+  $lights  = $g->lightest_vertices;
+
+  $x = $g->max_weight;  # Equal to the $heavies.
+  $y = $g->min_weight;  # Equal to the $lights.
+
+  # Call any methods of the inherited Graph module.
+  @v = $g->vertices;
+  @v = $g->successors($p);  # etc.
+  $z = $g->MST_Kruskal;
+  $z = $g->APSP_Floyd_Warshall;
+  $z = $g->MST_Prim($p);
 
 =head1 ABSTRACT
 
@@ -300,49 +346,55 @@ A weighted graph implementation
 
 =head1 DESCRIPTION
 
-A C<Graph::Weighted> object represents a subclass of C<Graph> with 
-weighted attributes that are taken from a 2D matrix (HoH or NxN LoL) 
-of numerical values.
+A C<Graph::Weighted> object represents a subclass of 
+C<Graph::Directed> with weighted attributes that are taken from a 
+two dimensional matrix (HoH or NxN LoL) of numerical values.
+
+This module can also load the matrix portions of C<Math::Matrix>, 
+C<Math::MatrixReal>, and C<Math::MatrixBool> objects.
 
 Initially, the weights of the vertices are set to the sum of their 
-outgoing edge weights.  This is mutable, however, and can be set to 
+outgoing edge weights.  This is mutable, however, and can be reset to 
 any value desired, after initialization, with the C<vertex_weight> 
-method.
+and C<edge_weight> methods.
 
 =head1 PUBLIC METHODS
 
 =over 4
 
-=item new HASH
+=item new %ARGUMENTS
 
 =over 4
 
-=item debug 0 | 1
+=item debug => 0 | 1
 
 Flag to invoke verbose mode while processing.  Defaults to zero.
 
-=item zero_edges 0 | 1
+=item zero_edges => 0 | 1
 
 Flag to add edges between vertices with a weight of zero.  Defaults to 
 zero.
 
-=item data HASHREF | ARRAYREF
+=item data => $HASHREF | $ARRAYREF | $OBJECT
 
-Two dimensional hash or (2D, square) array reference to use for 
-vertices and weighted edges.
+Two dimensional hash, (NxN) array, or known object reference to use 
+for vertices and weighted edges.
+
+The known objects are C<Math::Matrix>, C<Math::MatrixReal>, and 
+C<Math::MatrixBool>.
 
 =back
 
-=item reset_graph
+=item load $HASHREF | $ARRAYREF | $OBJECT
 
-Erase the graph's vertices, edges and attributes.
+Turn the given two dimensional hash, (NxN) array, or known object 
+reference into the vertices and weighted edges of a 
+C<Graph::Directed> object.
 
-=item load HASHREF | ARRAYREF
+The known objects are C<Math::Matrix>, C<Math::MatrixReal>, and 
+C<Math::MatrixBool>.
 
-Turn the given two dimensional hash or (2D, square) array reference 
-into the vertices and weighted edges of a C<Graph> object.
-
-=item matrix
+=item data
 
 Return the two dimensional hash used for vertices and weighted edges.
 
@@ -350,23 +402,30 @@ Return the two dimensional hash used for vertices and weighted edges.
 
 Get the total weight of the graph, by summing all the vertex weights.
 
-=item vertex_weight SCALAR [, SCALAR]
+=item vertex_weight $VERTEX [, $WEIGHT]
 
 Return the weight of a vertex.  This method can also be used to set 
 the vertex weight, if a second argument is provided.
 
+(The vertices are just the keys of the data, not some glorified 
+object.)
+
 When the second argument is provided, the weight it represents is 
 distributed evenly to the vertex's outgoing edges, and the total 
 weight of the entire graph is adjusted accordingly.
 
-=item edge_weight SCALAR, SCALAR [, SCALAR]
+=item edge_weight $VERTEX, $NEIGHBOR [, $WEIGHT]
 
-Return the weight of an edge.  This method can also be used to set
-the edge weight, if a third argument is provided.
+Return the weight of an edge between the two given vertices.  This 
+method can also be used to set the edge weight, if a third argument 
+is provided.
+
+(The vertices are just the keys of the data, not some glorified 
+object.)
 
 When the third argument is provided, the weight it represents is used
 to replace the weight of the edge between the vertex (first argument)
-and it's neighbor (second argument).  Lastly, the total weight of the 
+and it's successor (second argument).  Lastly, the total weight of the 
 entire graph and the weight of the vertex are adjusted accordingly.
 
 =item heaviest_vertices
@@ -377,13 +436,21 @@ Return the array reference of vertices with the most weight.
 
 Return the array reference of vertices with the least weight.
 
+=item max_weight
+
+Return the weight of the heaviest vertices.
+
+=item min_weight
+
+Return the weight of the lightest vertices.
+
 =back
 
 =head1 PRIVATE METHODS
 
 =over 4
 
-=item _debug ARRAY
+=item _debug @STUFF
 
 Print the contents of the argument array with a newline appended.
 
@@ -391,23 +458,19 @@ Print the contents of the argument array with a newline appended.
 
 =head1 SEE ALSO
 
-C<Graph>
+C<Graph::Base>
 
 =head1 TO DO
 
-Argh!  Find and purge the totally annoying, strong coupling side 
-effect in the C<graph_weight> method.
-
-Add attribute aware C<Graph> method tests to the test suite and 
-SYNOPSIS section.
-
 Handle clusters of vertices and sub-graphs.
 
-Handle C<Math::MatrixReal> objects.
+Handle "capacity graphs" as detailed in the C<Graph::Base> module?
+
+Handle Math::MatrixSparse and PDL::Matrix objects?
 
 =head1 AUTHOR
 
-Gene Boggs E<lt>cpan@ology.netE<gt>
+Gene Boggs E<lt>gene@cpan@.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
